﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;

namespace howto_trim_image
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The original and processed image.
        private Bitmap OriginalImage = null, CurrentImage = null;

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                SaveImage(CurrentImage, sfdImage.FileName);
            }
        }

        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                OriginalImage = LoadBitmapUnlocked(ofdImage.FileName);
                ProcessImage();

                mnuFileSaveAs.Enabled = true;
                btnGo.Enabled = true;
            }
        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            Refresh();

            ProcessImage();

            Cursor = Cursors.Default;
        }

        private void ProcessImage()
        {
            int margin = int.Parse(txtMargin.Text);
            CurrentImage = TrimImage(OriginalImage, margin);
            picImage.Image = CurrentImage;
        }

        // Trim the image to its non-white pixels plus a margin.
        private Bitmap TrimImage(Bitmap image, int margin)
        {
            // Make a Bitmap32.
            Bitmap32 bm32 = new Bitmap32(image);
            bm32.LockBitmap();

            // Find the pixel bounds.
            Rectangle src_rect = ImageBounds(bm32);
            bm32.UnlockBitmap();

            // Copy the non-white area.
            int wid = src_rect.Width + 2 * margin;
            int hgt = src_rect.Height + 2 * margin;
            Bitmap bm = new Bitmap(wid, hgt);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                gr.Clear(Color.White);
                Rectangle dest_rect = new Rectangle(
                    margin, margin, src_rect.Width, src_rect.Height);
                gr.DrawImage(image, dest_rect, src_rect, GraphicsUnit.Pixel);
            }

            return bm;
        }

        // Get the image's bounds.
        private Rectangle ImageBounds(Bitmap32 bm32)
        {
            // ymin.
            int ymin = bm32.Height - 1;
            for (int y = 0; y < bm32.Height; y++)
            {
                if (!RowIsWhite(bm32, y))
                {
                    ymin = y;
                    break;
                }
            }

            // ymax.
            int ymax = 0;
            for (int y = bm32.Height - 1; y >= ymin; y--)
            {
                if (!RowIsWhite(bm32, y))
                {
                    ymax = y;
                    break;
                }
            }

            // xmin.
            int xmin = bm32.Width - 1;
            for (int x = 0; x < bm32.Width; x++)
            {
                if (!ColumnIsWhite(bm32, x))
                {
                    xmin = x;
                    break;
                }
            }

            // xmax.
            int xmax = 0;
            for (int x = bm32.Width - 1; x >= xmin; x--)
            {
                if (!ColumnIsWhite(bm32, x))
                {
                    xmax = x;
                    break;
                }
            }

            // Build the rectangle.
            return new Rectangle(xmin, ymin,
                xmax - xmin + 1, ymax - ymin + 1);
        }

        // Return true if this row is all white.
        private bool RowIsWhite(Bitmap32 bm32, int y)
        {
            byte r, g, b, a;
            for (int x = 0; x < bm32.Width; x++)
            {
                bm32.GetPixel(x, y, out r, out g, out b, out a);
                if ((r != 255) || (g != 255) || (b != 255)) return false;
            }
            return true;
        }

        // Return true if this column is all white.
        private bool ColumnIsWhite(Bitmap32 bm32, int x)
        {
            byte r, g, b, a;
            for (int y = 0; y < bm32.Height; y++)
            {
                bm32.GetPixel(x, y, out r, out g, out b, out a);
                if ((r != 255) || (g != 255) || (b != 255)) return false;
            }
            return true;
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }
    }
}
